﻿Type=Class
Version=2.71
@EndOfDesignText@
'Class module
Sub Class_Globals
	
	Type CellImageView(CellImage As ImageView, CellCan As Canvas, CellCol As Int, CellRow As Int, TileAudit As Int, Exits(8) As Int, TileRotation As Int)
	
	
	' PRIVATE members	
	Private moParentAct As Object
	Private msBaseEvent As String 
	Private moHostingPanel As Panel
	Private moHostCan As Canvas 
	Private miBoardWidth As Int 
	Private miBoardHeight As Int 
	Private miXOffset As Int
	Private miYOffset As Int 
	Private miCellHeight As Int
	Private miCellWidth As Int 
	Private miXCellCount As Int 
	Private miYCellCount As Int
	Private miCellBorder As Int
	Private mfZoomFactor As Float 
	' cell values adjusted for zoom	
	Private miZoomWidth As Int 
	Private miZoomHeight As Int 
	Private miZoomOffsetX As Int 
	Private miZoomOffsetY As Int 
	Private miZoomBorder As Int 
	
	Private moCells As List             ' a LIST of CellImageView
End Sub


Public Sub Initialize(oCallingAct As Object, sBaseEventname As String, oHostingPanel As Panel)
	moParentAct = oCallingAct
	msBaseEvent = sBaseEventname
	moHostingPanel = oHostingPanel
	
	miXCellCount = 8
	miYCellCount = 8
	miCellWidth = 100dip
	miCellHeight = 100dip
	miCellBorder = 2dip
	
	moCells.Initialize 	
	
	' create and place cell image views
	InitializeImageviews
End Sub

Public Sub ResetZoom
	moHostingPanel.Width = miBoardWidth * mfZoomFactor
	moHostingPanel.Height = miBoardHeight * mfZoomFactor
	moHostingPanel.Invalidate 
	DoEvents
	' resize/position imageviews
	ResetCellPositions	
End Sub 

Public Sub GetZoom() As Float
	Return mfZoomFactor
End Sub 

Public Sub SetZoom(fNewZoom As Float)
	mfZoomFactor = fNewZoom
	ResetZoom 
End Sub 




Private Sub RefreshNeeded
	' let the owing activity know we changed something 
	If msBaseEvent <> "" Then
		Try
			If SubExists(moParentAct, msBaseEvent & "_Paint") Then
				CallSub(moParentAct, msBaseEvent & "_Paint")
			End If
		Catch
			LogColor("Error calling _Paint event in owning activity: " & LastException.Message, Colors.Magenta)
		End Try
	End If 		
End Sub 





#Region "  Setup ImageView Array on Panel "
Sub InitializeImageviews
	Dim piRow As Int 
	Dim piCol As Int 

	' calculate the 100% board width / height 
	'    Each square is 'miCellWidth' wide with a "border" of 'miCellBorder' on the left and right
	miBoardWidth = ((miCellBorder * 2) + miCellWidth) * miXCellCount 
	'  This gives us the amount required for just each image view so now add outside "border" of 1/2 the cell width on each side (which equals one whole cell)
	miBoardWidth = miBoardWidth + miCellWidth
	'  Now dow the same thing for the height values
	miBoardHeight = ((miCellBorder * 2) + miCellHeight) * miYCellCount 
	miBoardHeight = miBoardHeight + miCellHeight
	' this gives us 100% width and height
	
	' figure x/y offset where the first cell will be placed
	miXOffset = miCellWidth / 2
	miYOffset = miCellHeight / 2

	' make our hosting panel match 100%
	moHostingPanel.Width = miBoardWidth
	moHostingPanel.Height= miBoardHeight
	mfZoomFactor = 1.0
	DoEvents
	' save off the canvas
	moHostCan.Initialize(moHostingPanel)

	' Create cols * rows worth of imageviews
	For piRow = 0 To miYCellCount - 1
		For piCol = 0 To miXCellCount - 1
			Dim poImv As ImageView 
			poImv.Initialize("imvCell")
			moHostingPanel.AddView(poImv, 0, 0, miCellWidth, miCellWidth)
			poImv.Gravity = Gravity.FILL 
			' create a blank background
			Dim poBmp As Bitmap 
			Dim poCan As Canvas 
			poBmp.InitializeMutable(miCellWidth, miCellHeight)
			poCan.Initialize2(poBmp)
			poCan.DrawColor(Colors.RGB(200, 100, 0))
			' draw a number
			poCan.DrawText("(" & piCol & "," & piRow & ")", 10dip, 20dip, Typeface.default, 14, Colors.White, "LEFT")
			poImv.SetBackgroundImage(poBmp)
			Dim poNewCell As CellImageView 
			poNewCell.Initialize 
			poNewCell.CellImage = poImv
			poNewCell.CellCan = poCan 
			poNewCell.CellCol = piCol
			poNewCell.CellRow = piRow
			poNewCell.TileAudit = 0
			poNewCell.TileRotation = 0
			moCells.Add(poNewCell)
		Next 
	Next 
	' now resize and reposition all image views
	ResetCellPositions			
End Sub 


Private Sub ResetCellPositions
	Dim piIndex As Int
	Dim poRect As Rect 
	Dim poPoint As Point 
	
	' recalculate module vars to allow for zoom factor
	miZoomWidth = miCellWidth * mfZoomFactor
	miZoomHeight = miCellHeight * mfZoomFactor
	miZoomOffsetX = miXOffset * mfZoomFactor
	miZoomOffsetY = miYOffset * mfZoomFactor
	miZoomBorder = miCellBorder * mfZoomFactor	
	
	'clear entire panel with blue for border
	moHostCan.DrawColor(Colors.Blue)

	' fill inner panel with yellow as contrast for cell "borders"
	poRect.Initialize(miZoomOffsetX, miZoomOffsetY, moHostingPanel.Width - miZoomOffsetX, moHostingPanel.Height - miZoomOffsetY)
	moHostCan.DrawRect(poRect, Colors.Yellow, True, 1dip)

	' position each imageview	
	For piIndex = 0 To moCells.Size - 1
		Dim poCell As CellImageView 
		poCell = moCells.Get(piIndex)
		poPoint = GetCellPointPX(poCell.CellCol, poCell.CellRow, True)
		poCell.CellImage.SetLayout(poPoint.X, poPoint.Y, miZoomWidth, miZoomHeight)
	Next 
	
	' let hosting activity know something has changed	
	RefreshNeeded
End Sub 


Private Sub GetCellPointPX(CellCol As Int, CellRow As Int, InnerPoint As Boolean) As Point
	Dim poPoint As Point

	poPoint.Initialize 
	
	If InnerPoint Then
		poPoint.X = miZoomOffsetX + (CellCol * (miZoomWidth + (miZoomBorder * 2))) + miZoomBorder
		poPoint.Y = miZoomOffsetY + (CellRow * (miZoomHeight + (miZoomBorder * 2))) + miZoomBorder
	Else
		poPoint.X = miZoomOffsetX + (CellCol * (miZoomWidth + (miZoomBorder * 2))) 
		poPoint.Y = miZoomOffsetY + (CellRow * (miZoomHeight + (miZoomBorder * 2))) 
	End If 

	Return poPoint
End Sub 




#End Region 









