﻿Type=StaticCode
Version=2.71
@EndOfDesignText@
'Code module
'  THIS MODULE REQUIRES the following libraries:
'       Core
'       Reflection
'       ByteConverter
'
'  http://www.basic4ppc.com/forum/basic4android-getting-started-tutorials/28950-compiler-warnings.html#post167904
#IgnoreWarnings: 9, 10, 12

Sub Process_Globals
	Public goBC As ByteConverter 
	Type ShiftData(PanelView As View, Width As Int, Height As Int)	
	Type Point(X As Int, Y As Int)
End Sub



#Region "  String Functions "

'#######################
'### STRING FUNCTIONS
'#######################

Sub Left(Text As String, Length As Long)As String   
	If Length>Text.Length Then 
		Length=Text.Length   
	End If 
	Return Text.SubString2(0, Length)
End Sub

Sub Right(Text As String, Length As Long) As String   
	If Length>Text.Length Then 
		Length=Text.Length
	End If
	Return Text.SubString(Text.Length-Length)
End Sub

Sub Mid(Text As String, Start As Int, Length As Int) As String   
	Return Text.SubString2(Start-1,Start+Length-1)
End Sub

Sub CountChars(StringToScan As String, CharToCount As String) As Int
	Dim piIndex As Int
	Dim piCount As Int
	
	For piIndex = 0 To StringToScan.Length - 1 
		If StringToScan.SubString2(piIndex, piIndex + 1) = CharToCount Then
			piCount = piCount + 1
		End If 
	Next
	
	Return piCount
End Sub 

Public Sub ReverseText(sText As String) As String
	Dim piIndex As Int
	Dim poSB As StringBuilder 

	If sText.Length < 2 Then
		Return sText
	Else
		poSB.Initialize
		For piIndex = sText.Length - 1 To 0 Step - 1
			poSB.Append(sText.SubString2(piIndex, piIndex + 1))
		Next 
		Return poSB.ToString 
	End If 
End Sub 

#End Region 

#Region "  Graphics Functions "
'#######################
'### GRAPHICS FUNCTIONS
'#######################

Sub ResizeBitmap(oSource As Bitmap, NewWidth As Int, NewHeight As Int) As Bitmap 
	Dim poResized As Bitmap
	Dim poCan As Canvas 
	Dim poSrcRect As Rect
	Dim poDestRect As Rect

	' resize bitmap 
	poResized.InitializeMutable(NewWidth, NewHeight)
	poCan.Initialize2(poResized)	
	poSrcRect.Initialize(0, 0, oSource.Width, oSource.Height) 'incoming size is 32x32
	poDestRect.Initialize(0, 0, NewWidth, NewHeight)
	poCan.DrawBitmap(oSource, poSrcRect, poDestRect)

	Return poResized
End Sub 
	
Sub CreateScaledBitmap(Original As Bitmap, Width As Int, Height As Int, Filter As Boolean) As Bitmap
    Dim r As Reflector
    Dim b As Bitmap
    b = r.RunStaticMethod("android.graphics.Bitmap", "createScaledBitmap", _
        Array As Object(Original, Width, Height, Filter), _
        Array As String("android.graphics.Bitmap", "java.lang.int", "java.lang.int", "java.lang.boolean"))
    Return b
End Sub


' Return the High Contract color (black or white) according to the selected color for background
Sub ContrastColor(C As Int) As Int
    Dim D As Int
    Dim a As Double
    Dim argb() As Int
    argb = GetARGB(C)
         a = 1 - ( 0.299 * argb(1) + 0.587 * argb(2) + 0.114 * argb(3))/255
    If a < 0.5 Then
        D = 0 ' bright Colors - black font
    Else
        D = 255 ' dark Colors - white font
    End If
    Return  Colors.RGB(D, D, D)
End Sub

' Returns an array of Int that describes the color passed in.<b />
' (0) = Alpha value, (1) = Red, (2) = Green, (3) = Blue
Sub GetARGB(Color As Int) As Int()
    Dim res(4) As Int
    res(0) = Bit.UnsignedShiftRight(Bit.AND(Color, 0xff000000), 24)
    res(1) = Bit.UnsignedShiftRight(Bit.AND(Color, 0xff0000), 16)
    res(2) = Bit.UnsignedShiftRight(Bit.AND(Color, 0xff00), 8)
    res(3) = Bit.AND(Color, 0xff)
    Return res
End Sub

Sub GetViewsColor(v As View) As Int
	Dim destRect As Rect    
	Dim cvs As Canvas 
    Dim bmp As Bitmap    
	Try
		bmp.InitializeMutable(v.Width, v.Height)    
		cvs.Initialize2(bmp)	
		
		destRect.Initialize(0, 0, cvs.Bitmap.Width, cvs.Bitmap.Height)    
		cvs.DrawDrawable(v.Background, destRect)
		Return cvs.Bitmap.GetPixel((cvs.Bitmap.Width / 2), (cvs.Bitmap.Height / 2))
	Catch
		Return 0
	End Try
End Sub
		
Sub UnsignBytes(BytesIn() As Byte) As Byte()
	Dim piIndex As Int
	Dim NewBytes(BytesIn.Length) As Byte
	
	For piIndex = 0 To BytesIn.Length - 1 
		NewBytes(piIndex) = Bit.AND(BytesIn(piIndex), 0xff)
	Next 
	
	Return NewBytes
End Sub 

Sub BitmapFromByteArray(oBytes() As Byte) As Bitmap
	Dim poIS As InputStream
    Dim poBmp As Bitmap

	poIS.InitializeFromBytesArray(oBytes, 0, oBytes.Length)
    poBmp.Initialize2(poIS)
    poIS.Close

	Return poBmp
End Sub 

Sub SaveBitmapToDisk(oBitmap As Bitmap, sDirectory As String, sFileName As String) As Boolean
	
	Try
		Dim poOut As OutputStream 
		poOut = File.OpenOutput(sDirectory, sFileName, False)
		oBitmap.WriteToStream(poOut, 100, "PNG")
		poOut.Close
		Return True 
	Catch
		LogColor("### Error saving bitmap file in " & sDirectory & " as " & sFileName & ": " & LastException.Message, Colors.Red)
	End Try 
	Return False
End Sub 


#End Region 

#Region "  Utility Functions "
'#######################
'### UTILITY FUNCTIONS
'#######################

Sub GetDeviceId As String    
	Dim r As Reflector    
	Dim Api As Int    
	
	Api = r.GetStaticField("android.os.Build$VERSION", "SDK_INT")    
	If Api < 9 Then        
		'Old device        
		If File.Exists(File.DirInternal, "__id") Then            
			Return File.ReadString(File.DirInternal, "__id")
		Else            
			Dim id As Int            
			id = Rnd(0x10000000, 0x7FFFFFFF)            
			File.WriteString(File.DirInternal, "__id", id)            
			Return id        
		End If    
	Else        
		'New device        
		Return r.GetStaticField("android.os.Build", "SERIAL")    
	End If
End Sub


'  Modifies the passed label to have "..." in it.  Modes are "NONE", "START", "MIDDLE", "END", "MARQUEE"
Sub SetEllipsize(TextView As Label, Mode As String)
	Dim r As Reflector
	r.Target = TextView    
	r.RunMethod2("setLines", 1, "java.lang.int")    
	r.RunMethod2("setHorizontallyScrolling", True, "java.lang.boolean")     
	r.RunMethod2("setEllipsize", Mode, "android.text.TextUtils$TruncateAt")
	r.RunMethod2("setSelected", True, "java.lang.boolean")
End Sub

Sub SoundDefaultBeep
	Try
		Dim Obj1 As Reflector
		Obj1.Target = Obj1.CreateObject("android.provider.Settings$System")
		Obj1.Target = Obj1.GetField("DEFAULT_NOTIFICATION_URI")
		Dim args(2) As Object
		args(0) = Obj1.GetContext
		args(1) = Obj1.Target
		Dim types(2) As String
		types(0) = "android.content.Context"
		types(1) = "android.net.Uri"
		Obj1.Target = Obj1.RunStaticMethod("android.media.MediaPlayer", "create", args, types)
		Obj1.RunMethod("start")
	Catch
		LogColor("## RTE attempting to default uri beep intent: " & LastException.Message, Colors.Magenta)
	End Try
End Sub 

Sub GenerateGUID() As String
	Dim r As Reflector
 	r.Target = r.RunStaticMethod("java.util.UUID", "randomUUID", Null, Null)
 	Return r.RunMethod("toString")
End Sub

#End Region 

#Region "  View Functions "
'#######################
'### VIEW FUNCTIONS
'#######################


' Inserts a new panel into a scrollview that contains a collection of evenly-spaced panels
'   <b>oScv</b> = The scrollview<br/>
'   <b>iTargetIndex</b> = the target position to insert the supplied panel 
'   <b>bInsertBefore</b> =  <i>True</i> = insert before supplied index (new panel index value will be iTargeIndex) OR <i>False</i> = insert after the supplied index (new panel index value will be 1 more than iTargetIndex)
'   <b>oPanelToInsert</b> = the panel that will be inserted into the scrollview
'   <b>iPanelHeight</b> - the desired height of the panel you are inserting
'   <b>iInterPanelGap</b> = the number DIP spacing to place between panels
Sub ScvInsertPanel(oScv As ScrollView, bInsertBefore As Boolean, iTargetIndex As Int, oPanelToInsert As Panel, iPanelHeight As Int, iInterPanelGap As Int)
	Dim piIndex As Int
	Dim piY As Int
	Dim poShift As List
	Dim piEndTarget As Int 
	
	If iTargetIndex > oScv.Panel.NumberOfViews - 1 Then
		' just tack on to the bottom
		piY = 0
		For piIndex = 0 To oScv.Panel.NumberOfViews - 1
			If oScv.Panel.GetView(piIndex).Top > piY Then
				piY = oScv.Panel.GetView(piIndex).Top + oScv.Panel.GetView(piIndex).Height 
			End If
		Next 
		' add our gap
		piY = piY + iInterPanelGap
		' insert new panel
		oScv.Panel.AddView(oPanelToInsert, 0, piY, oScv.Panel.Width, iPanelHeight)
		piY = piY + iInterPanelGap + iPanelHeight
	Else
		poShift.Initialize
		If bInsertBefore = True Then
			piEndTarget = iTargetIndex
		Else
			piEndTarget = iTargetIndex + 1
		End If 
		For piIndex = oScv.Panel.NumberOfViews - 1 To piEndTarget Step -1
			Dim poPan As ShiftData
			poPan.Height = oScv.Panel.GetView(piIndex).Height 
			poPan.Width = oScv.Panel.GetView(piIndex).Width 
			poPan.PanelView = oScv.Panel.GetView(piIndex)
			poShift.Add(poPan)
			oScv.Panel.RemoveViewAt(piIndex)
		Next 
		' get height of panel now
		piY = 0
		If oScv.Panel.NumberOfViews = 1 Then
			piY = oScv.Panel.GetView(0).Top + oScv.Panel.GetView(0).Height 
		Else
			For piIndex = 0 To oScv.Panel.NumberOfViews - 1
				If oScv.Panel.GetView(piIndex).Top > piY Then
					piY = oScv.Panel.GetView(piIndex).Top + oScv.Panel.GetView(piIndex).Height 
				End If
			Next 
		End If 
		' add our gap if we're not inserting at zero
		If piY > 0 Then
			piY = piY + iInterPanelGap
		End If 
		' insert new panel
		oScv.Panel.AddView(oPanelToInsert, 0, piY, oScv.Panel.Width, iPanelHeight)
		piY = piY + iInterPanelGap + iPanelHeight
		' add other panels back in
		For piIndex = poShift.Size - 1 To 0 Step - 1
			Dim poOld As ShiftData
			poOld = poShift.Get(piIndex)
			oScv.Panel.AddView(poOld.PanelView, 0, piY, poOld.Width, poOld.Height)
			piY = piY + iInterPanelGap + poOld.Height 
		Next 
	End If 
	oScv.Panel.Height = piY 
End Sub 




#End Region 
